/*

The MIT License (MIT)

Copyright (c) 2014 Juan Cazala - juancazala.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE



********************************************************************************************
                                         SYNAPTIC
********************************************************************************************

Synaptic is a javascript neural network library for node.js and the browser, its generalized
algorithm is architecture-free, so you can build and train basically any type of first order
or even second order neural network architectures.

http://en.wikipedia.org/wiki/Recurrent_neural_network#Second_Order_Recurrent_Neural_Network

The library includes a few built-in architectures like multilayer perceptrons, multilayer
long-short term memory networks (LSTM) or liquid state machines, and a trainer capable of
training any given network, and includes built-in training tasks/tests like solving an XOR,
passing a Distracted Sequence Recall test or an Embeded Reber Grammar test.

The algorithm implemented by this library has been taken from Derek D. Monner's paper:


A generalized LSTM-like training algorithm for second-order recurrent neural networks
http://www.overcomplete.net/papers/nn2012.pdf

There are references to the equations in that paper commented through the source code.


********************************************************************************************/
!function t(e,i,n){function r(s,a){if(!i[s]){if(!e[s]){var c="function"==typeof require&&require;if(!a&&c)return c(s,!0);if(o)return o(s,!0);var h=new Error("Cannot find module '"+s+"'");throw h.code="MODULE_NOT_FOUND",h}var u=i[s]={exports:{}};e[s][0].call(u.exports,function(t){var i=e[s][1][t];return r(i?i:t)},u,u.exports,t,e,i,n)}return i[s].exports}for(var o="function"==typeof require&&require,s=0;s<n.length;s++)r(n[s]);return r}({1:[function(t,e,i){var n={Neuron:t("./neuron"),Layer:t("./layer"),Network:t("./network"),Trainer:t("./trainer"),Architect:t("./architect")};"undefined"!=typeof define&&define.amd&&define([],function(){return n}),"undefined"!=typeof e&&e.exports&&(e.exports=n),"object"==typeof window&&(!function(){var t=window.synaptic;n.ninja=function(){return window.synaptic=t,n}}(),window.synaptic=n)},{"./architect":2,"./layer":3,"./network":4,"./neuron":5,"./trainer":6}],2:[function(t,e,i){var n=t("./layer"),r=t("./network"),o=t("./trainer"),s={Perceptron:function(){var t=Array.prototype.slice.call(arguments);if(t.length<3)throw"Error: not enough layers (minimum 3) !!";var e=t.shift(),i=t.pop(),r=t,s=new n(e),a=[],c=new n(i),h=s;for(level in r){var u=r[level],p=new n(u);a.push(p),h.project(p),h=p}h.project(c),this.set({input:s,hidden:a,output:c}),this.trainer=new o(this)},LSTM:function(){var t=Array.prototype.slice.call(arguments);if(t.length<3)throw"Error: not enough layers (minimum 3) !!";var e=t.pop(),i={peepholes:n.connectionType.ALL_TO_ALL,hiddenToHidden:!1,outputToHidden:!1,outputToGates:!1,inputToOutput:!0};if("number"!=typeof e){var r=t.pop();e.hasOwnProperty("peepholes")&&(i.peepholes=e.peepholes),e.hasOwnProperty("hiddenToHidden")&&(i.hiddenToHidden=e.hiddenToHidden),e.hasOwnProperty("outputToHidden")&&(i.outputToHidden=e.outputToHidden),e.hasOwnProperty("outputToGates")&&(i.outputToGates=e.outputToGates),e.hasOwnProperty("inputToOutput")&&(i.inputToOutput=e.inputToOutput)}else var r=e;var s=t.shift(),a=t,c=new n(s),h=[],u=new n(r),p=null;for(var f in a){var l=a[f],v=new n(l).set({bias:1}),d=new n(l).set({bias:1}),g=new n(l),y=new n(l).set({bias:1});h.push(v),h.push(d),h.push(g),h.push(y);var m=c.project(g);if(c.project(v),c.project(d),c.project(y),null!=p){var w=p.project(g);p.project(v),p.project(d),p.project(y)}var T=g.project(u),b=g.project(g);i.hiddenToHidden&&g.project(g,n.connectionType.ALL_TO_ELSE),i.outputToHidden&&u.project(g),i.outputToGates&&(u.project(v),u.project(y),u.project(d)),g.project(v,i.peepholes),g.project(d,i.peepholes),g.project(y,i.peepholes),v.gate(m,n.gateType.INPUT),d.gate(b,n.gateType.ONE_TO_ONE),y.gate(T,n.gateType.OUTPUT),null!=p&&v.gate(w,n.gateType.INPUT),p=g}i.inputToOutput&&c.project(u),this.set({input:c,hidden:h,output:u}),this.trainer=new o(this)},Liquid:function(t,e,i,r,s){for(var a=new n(t),c=new n(e),h=new n(i),u=c.neurons(),p=[],f=0;r>f;f++){var l=Math.random()*u.length|0,v=Math.random()*u.length|0,d=u[l].project(u[v]);p.push(d)}for(var g=0;s>g;g++){var y=Math.random()*u.length|0,d=Math.random()*p.length|0;u[y].gate(p[d])}a.project(c),c.project(h),this.set({input:a,hidden:[c],output:h}),this.trainer=new o(this)},Hopfield:function(t){var e=new n(t),i=new n(t);e.project(i,n.connectionType.ALL_TO_ALL),this.set({input:e,hidden:[],output:i});var r=new o(this),a=s.Hopfield.prototype;a.learn=a.learn||function(t){var e=[];for(var i in t)e.push({input:t[i],output:t[i]});return r.train(e,{iterations:5e5,error:5e-5,rate:1})},a.feed=a.feed||function(t){var e=this.activate(t),t=[];for(var i in e)t[i]=e[i]>.5?1:0;return t}}};for(var a in s)s[a].prototype=new r,s[a].prototype.constructor=s[a];e&&(e.exports=s)},{"./layer":3,"./network":4,"./trainer":6}],3:[function(t,e,i){function n(t,e){for(this.size=0|t,this.list=[],this.label=e||null,this.connectedTo=[];t--;){var i=new r;this.list.push(i)}}var r=t("./neuron");n.prototype={activate:function(t){var e=[];if("undefined"!=typeof t){if(t.length!=this.size)throw"INPUT size and LAYER size must be the same to activate!";for(var i in this.list){var n=this.list[i],r=n.activate(t[i]);e.push(r)}}else for(var i in this.list){var n=this.list[i],r=n.activate();e.push(r)}return e},propagate:function(t,e){if("undefined"!=typeof e){if(e.length!=this.size)throw"TARGET size and LAYER size must be the same to propagate!";for(var i=this.list.length-1;i>=0;i--){var n=this.list[i];n.propagate(t,e[i])}}else for(var i=this.list.length-1;i>=0;i--){var n=this.list[i];n.propagate(t)}},project:function(e,i,r){if(e instanceof t("./network")&&(e=e.layers.input),!(e instanceof n))throw"Invalid argument, you can only project connections to LAYERS and NETWORKS!";return this.connected(e)?void 0:new n.connection(this,e,i,r)},gate:function(t,e){if(e==n.gateType.INPUT){if(t.to.size!=this.size)throw"GATER layer and CONNECTION.TO layer must be the same size in order to gate!";for(var i in t.to.list){var r=t.to.list[i],o=this.list[i];for(var s in r.connections.inputs){var a=r.connections.inputs[s];a.ID in t.connections&&o.gate(a)}}}else if(e==n.gateType.OUTPUT){if(t.from.size!=this.size)throw"GATER layer and CONNECTION.FROM layer must be the same size in order to gate!";for(var i in t.from.list){var r=t.from.list[i],o=this.list[i];for(var c in r.connections.projected){var a=r.connections.projected[c];a.ID in t.connections&&o.gate(a)}}}else if(e==n.gateType.ONE_TO_ONE){if(t.size!=this.size)throw"The number of GATER UNITS must be the same as the number of CONNECTIONS to gate!";for(var i in t.list){var o=this.list[i],a=t.list[i];o.gate(a)}}t.gatedfrom.push({layer:this,type:e})},selfconnected:function(){for(var t in this.list){var e=this.list[t];if(!e.selfconnected())return!1}return!0},connected:function(t){var e=0;for(var i in this.list)for(var r in t.list){var o=this.list[i],s=t.list[r],a=o.connected(s);"projected"==a.type&&e++}if(e==this.size*t.size)return n.connectionType.ALL_TO_ALL;e=0;for(var c in this.list){var o=this.list[c],s=t.list[c],a=o.connected(s);"projected"==a.type&&e++}return e==this.size?n.connectionType.ONE_TO_ONE:void 0},clear:function(){for(var t in this.list){var e=this.list[t];e.clear()}},reset:function(){for(var t in this.list){var e=this.list[t];e.reset()}},neurons:function(){return this.list},add:function(t){this.neurons[t.ID]=t||new r,this.list.push(t),this.size++},set:function(t){t=t||{};for(var e in this.list){var i=this.list[e];t.label&&(i.label=t.label+"_"+i.ID),t.squash&&(i.squash=t.squash),t.bias&&(i.bias=t.bias)}return this}},n.connection=function(t,e,i,r){if(this.ID=n.connection.uid(),this.from=t,this.to=e,this.selfconnection=e==t,this.type=i,this.connections={},this.list=[],this.size=0,this.gatedfrom=[],"undefined"==typeof this.type&&(this.type=t==e?n.connectionType.ONE_TO_ONE:n.connectionType.ALL_TO_ALL),this.type==n.connectionType.ALL_TO_ALL||this.type==n.connectionType.ALL_TO_ELSE)for(var o in this.from.list)for(var s in this.to.list){var a=this.from.list[o],c=this.to.list[s];if(this.type!=n.connectionType.ALL_TO_ELSE||a!=c){var h=a.project(c,r);this.connections[h.ID]=h,this.size=this.list.push(h)}}else if(this.type==n.connectionType.ONE_TO_ONE)for(var u in this.from.list){var a=this.from.list[u],c=this.to.list[u],h=a.project(c,r);this.connections[h.ID]=h,this.size=this.list.push(h)}t.connectedTo.push(this)},n.connectionType={},n.connectionType.ALL_TO_ALL="ALL TO ALL",n.connectionType.ONE_TO_ONE="ONE TO ONE",n.connectionType.ALL_TO_ELSE="ALL TO ELSE",n.gateType={},n.gateType.INPUT="INPUT",n.gateType.OUTPUT="OUTPUT",n.gateType.ONE_TO_ONE="ONE TO ONE",function(){var t=0;n.connection.uid=function(){return t++}}(),e&&(e.exports=n)},{"./network":4,"./neuron":5}],4:[function(t,e,i){function n(t){"undefined"!=typeof t&&(this.layers=t||{input:null,hidden:{},output:null},this.optimized=null)}var r=t("./neuron"),o=t("./layer");n.prototype={activate:function(t){if(this.optimized===!1){this.layers.input.activate(t);for(var e in this.layers.hidden)this.layers.hidden[e].activate();return this.layers.output.activate()}return null==this.optimized&&this.optimize(),this.optimized.activate(t)},propagate:function(t,e){if(this.optimized===!1){this.layers.output.propagate(t,e);var i=[];for(var n in this.layers.hidden)i.push(this.layers.hidden[n]);i.reverse();for(var n in i)i[n].propagate(t)}else null==this.optimized&&this.optimize(),this.optimized.propagate(t,e)},project:function(t,e,i){if(this.optimized&&this.optimized.reset(),t instanceof n)return this.layers.output.project(t.layers.input,e,i);if(t instanceof o)return this.layers.output.project(t,e,i);throw"Invalid argument, you can only project connections to LAYERS and NETWORKS!"},gate:function(t,e){this.optimized&&this.optimized.reset(),this.layers.output.gate(t,e)},clear:function(){this.restore();var t=this.layers.input,e=this.layers.output;t.clear();for(var i in this.layers.hidden){var n=this.layers.hidden[i];n.clear()}e.clear(),this.optimized&&this.optimized.reset()},reset:function(){this.restore();var t=this.layers.input,e=this.layers.output;t.reset();for(var i in this.layers.hidden){var n=this.layers.hidden[i];n.reset()}e.reset(),this.optimized&&this.optimized.reset()},optimize:function(){var t=this,e={},i=this.neurons();for(var n in i){for(var r=i[n].neuron,o=i[n].layer;r.neuron;)r=r.neuron;e=r.optimize(e,o)}for(var n in e.propagation_sentences)e.propagation_sentences[n].reverse();e.propagation_sentences.reverse();var s="";s+="var F = Float64Array ? new Float64Array("+e.memory+") : []; ";for(var n in e.variables)s+="F["+e.variables[n].id+"] = "+(e.variables[n].value||0)+"; ";s+="var activate = function(input){\n";for(var n in e.inputs)s+="F["+e.inputs[n]+"] = input["+n+"]; ";for(var a in e.activation_sentences)if(e.activation_sentences[a].length>0)for(var c in e.activation_sentences[a])s+=e.activation_sentences[a][c].join(" "),s+=e.trace_sentences[a][c].join(" ");s+=" var output = []; ";for(var n in e.outputs)s+="output["+n+"] = F["+e.outputs[n]+"]; ";s+="return output; }; ",s+="var propagate = function(rate, target){\n",s+="F["+e.variables.rate.id+"] = rate; ";for(var n in e.targets)s+="F["+e.targets[n]+"] = target["+n+"]; ";for(var a in e.propagation_sentences)for(var c in e.propagation_sentences[a])s+=e.propagation_sentences[a][c].join(" ")+" ";s+=" };\n",s+="var ownership = function(memoryBuffer){\nF = memoryBuffer;\nthis.memory = F;\n};\n",s+="return {\nmemory: F,\nactivate: activate,\npropagate: propagate,\nownership: ownership\n};",s=s.split(";").join(";\n");var h=new Function(s),u=h();u.data={variables:e.variables,activate:e.activation_sentences,propagate:e.propagation_sentences,trace:e.trace_sentences,inputs:e.inputs,outputs:e.outputs,check_activation:this.activate,check_propagation:this.propagate},u.reset=function(){t.optimized&&(t.optimized=null,t.activate=u.data.check_activation,t.propagate=u.data.check_propagation)},this.optimized=u,this.activate=u.activate,this.propagate=u.propagate},restore:function(){if(this.optimized){var t=this.optimized,e=function(){var e=Array.prototype.slice.call(arguments),i=e.shift(),n=e.pop(),r=n+"_";for(var o in e)r+=e[o]+"_";r+=i.ID;var s=t.memory,a=t.data.variables;return r in a?s[a[r].id]:0},i=this.neurons();for(var n in i){for(var r=i[n].neuron;r.neuron;)r=r.neuron;r.state=e(r,"state"),r.old=e(r,"old"),r.activation=e(r,"activation"),r.bias=e(r,"bias");for(var o in r.trace.elegibility)r.trace.elegibility[o]=e(r,"trace","elegibility",o);for(var s in r.trace.extended)for(var o in r.trace.extended[s])r.trace.extended[s][o]=e(r,"trace","extended",s,o)}for(var n in i){for(var r=i[n].neuron;r.neuron;)r=r.neuron;for(var a in r.connections.projected){var c=r.connections.projected[a];c.weight=e(c,"weight"),c.gain=e(c,"gain")}}}},neurons:function(){var t=[],e=this.layers.input.neurons(),i=this.layers.output.neurons();for(var n in e)t.push({neuron:e[n],layer:"input"});for(var r in this.layers.hidden){var o=this.layers.hidden[r].neurons();for(var n in o)t.push({neuron:o[n],layer:r})}for(var n in i)t.push({neuron:i[n],layer:"output"});return t},inputs:function(){return this.layers.input.size},outputs:function(){return this.layers.output.size},set:function(t){this.layers=t,this.optimized&&this.optimized.reset()},setOptimize:function(t){this.restore(),this.optimized&&this.optimized.reset(),this.optimized=t?null:!1},toJSON:function(t){this.restore();var e=this.neurons(),i=[],n=[],o={};for(var s in e){for(var a=e[s].neuron;a.neuron;)a=a.neuron;o[a.ID]=s;var c={trace:{elegibility:{},extended:{}},state:a.state,old:a.old,activation:a.activation,bias:a.bias,layer:e[s].layer};c.squash=a.squash==r.squash.LOGISTIC?"LOGISTIC":a.squash==r.squash.TANH?"TANH":a.squash==r.squash.IDENTITY?"IDENTITY":a.squash==r.squash.HLIM?"HLIM":null,i.push(c)}for(var s in e){for(var a=e[s].neuron;a.neuron;)a=a.neuron;for(var h in a.connections.projected){var u=a.connections.projected[h];n.push({from:o[u.from.ID],to:o[u.to.ID],weight:u.weight,gater:u.gater?o[u.gater.ID]:null})}a.selfconnected()&&n.push({from:o[a.ID],to:o[a.ID],weight:a.selfconnection.weight,gater:a.selfconnection.gater?o[a.selfconnection.gater.ID]:null})}return{neurons:i,connections:n}},toDot:function(t){var e="digraph nn {\n    rankdir = BT\n",i=[this.layers.input].concat(this.layers.hidden,this.layers.output);for(var n in i)for(var r in i[n].connectedTo){var o=i[n].connectedTo[r],s=o.to,a=o.size,c=i.indexOf(i[n]),h=i.indexOf(s);if(t){if(o.gatedfrom.length){var u="fake"+c+"_"+h;e+="    "+u+' [label = "", shape = point, width = 0.01, height = 0.01]\n',e+="    "+c+" -> "+u+" [label = "+a+", arrowhead = none]\n",e+="    "+u+" -> "+h+"\n"}else e+="    "+c+" -> "+h+" [label = "+a+"]\n";for(var p in o.gatedfrom){var f=o.gatedfrom[p].layer,l=(o.gatedfrom[p].type,i.indexOf(f));e+="    "+l+" -> "+u+" [color = blue]\n"}}else{e+="    "+c+" -> "+h+" [label = "+a+"]\n";for(var p in o.gatedfrom){var f=o.gatedfrom[p].layer,l=(o.gatedfrom[p].type,i.indexOf(f));e+="    "+l+" -> "+h+" [color = blue]\n"}}}return e+="}\n",{code:e,link:"https://chart.googleapis.com/chart?chl="+escape(e.replace("/ /g","+"))+"&cht=gv"}},standalone:function(){this.optimized||this.optimize();var t=this.optimized.data,e="function (input) {\n";for(var i in t.inputs)e+="F["+t.inputs[i]+"] = input["+i+"];\n";for(var n in t.activate)for(var r in t.activate[n])e+=t.activate[n][r]+"\n";e+="var output = [];\n";for(var i in t.outputs)e+="output["+i+"] = F["+t.outputs[i]+"];\n";e+="return output;\n}";var o=e.match(/F\[(\d+)\]/g),s=0,a={};for(var c in o){var h=o[c].match(/\d+/)[0];h in a||(a[h]=s++)}var u="F = {\n";for(var i in a)u+=a[i]+": "+this.optimized.memory[i]+",\n";return u=u.substring(0,u.length-2)+"\n};\n",u="var run = "+e.replace(/F\[(\d+)]/g,function(t){return"F["+a[t.match(/\d+/)[0]]+"]"}).replace("{\n","{\n"+u)+";\n",u+="return run",new Function(u)()},worker:function(){this.optimized||this.optimize();var t="var inputs = "+this.optimized.data.inputs.length+";\n";t+="var outputs = "+this.optimized.data.outputs.length+";\n",t+="var F = null;\n",t+="var activate = "+this.optimized.activate.toString()+";\n",t+="var propagate = "+this.optimized.propagate.toString()+";\n",t+="onmessage = function(e){\n",t+="F = e.data.memoryBuffer;\n",t+="if (e.data.action == 'activate'){\n",t+="if (e.data.input.length == inputs){\n",t+="postMessage( { action: 'activate', output: activate(e.data.input), memoryBuffer: F }, [F.buffer]);\n",t+="}\n}\nelse if (e.data.action == 'propagate'){\n",t+="propagate(e.data.rate, e.data.target);\n",t+="postMessage({ action: 'propagate', memoryBuffer: F }, [F.buffer]);\n",t+="}\n}\n";var e=new Blob([t]),i=window.URL.createObjectURL(e);return new Worker(i)},clone:function(){return n.fromJSON(this.toJSON())}},n.fromJSON=function(t){var e=[],i={input:new o,hidden:[],output:new o};for(var s in t.neurons){var a=t.neurons[s],c=new r;c.trace.elegibility={},c.trace.extended={},c.state=a.state,c.old=a.old,c.activation=a.activation,c.bias=a.bias,c.squash=a.squash in r.squash?r.squash[a.squash]:r.squash.LOGISTIC,e.push(c),"input"==a.layer?i.input.add(c):"output"==a.layer?i.output.add(c):("undefined"==typeof i.hidden[a.layer]&&(i.hidden[a.layer]=new o),i.hidden[a.layer].add(c))}for(var s in t.connections){var a=t.connections[s],h=e[a.from],u=e[a.to],p=a.weight,f=e[a.gater],l=h.project(u,p);f&&f.gate(l)}return new n(i)},e&&(e.exports=n)},{"./layer":3,"./neuron":5}],5:[function(t,e,i){function n(){this.ID=n.uid(),this.label=null,this.connections={inputs:{},projected:{},gated:{}},this.error={responsibility:0,projected:0,gated:0},this.trace={elegibility:{},extended:{},influences:{}},this.state=0,this.old=0,this.activation=0,this.selfconnection=new n.connection(this,this,0),this.squash=n.squash.LOGISTIC,this.neighboors={},this.bias=.2*Math.random()-.1}n.prototype={activate:function(t){if("undefined"!=typeof t)return this.activation=t,this.derivative=0,this.bias=0,this.activation;this.old=this.state,this.state=this.selfconnection.gain*this.selfconnection.weight*this.state+this.bias;for(var e in this.connections.inputs){var t=this.connections.inputs[e];this.state+=t.from.activation*t.weight*t.gain}this.activation=this.squash(this.state),this.derivative=this.squash(this.state,!0);var i=[];for(var n in this.trace.extended){var r=this.trace.extended[n],o=this.neighboors[n],s=o.selfconnection.gater==this?o.old:0;for(var a in this.trace.influences[o.ID])s+=this.trace.influences[o.ID][a].weight*this.trace.influences[o.ID][a].from.activation;i[o.ID]=s}for(var e in this.connections.inputs){var t=this.connections.inputs[e];this.trace.elegibility[t.ID]=this.selfconnection.gain*this.selfconnection.weight*this.trace.elegibility[t.ID]+t.gain*t.from.activation;for(var n in this.trace.extended){var r=this.trace.extended[n],o=this.neighboors[n],s=i[o.ID];r[t.ID]=o.selfconnection.gain*o.selfconnection.weight*r[t.ID]+this.derivative*this.trace.elegibility[t.ID]*s}}for(var c in this.connections.gated)this.connections.gated[c].gain=this.activation;return this.activation},propagate:function(t,e){var i=0,n="undefined"!=typeof e;if(n)this.error.responsibility=this.error.projected=e-this.activation;else{for(var r in this.connections.projected){var o=this.connections.projected[r],s=o.to;i+=s.error.responsibility*o.gain*o.weight}this.error.projected=this.derivative*i,i=0;for(var r in this.trace.extended){var s=this.neighboors[r],a=s.selfconnection.gater==this?s.old:0;for(var c in this.trace.influences[r])a+=this.trace.influences[r][c].weight*this.trace.influences[s.ID][c].from.activation;i+=s.error.responsibility*a}this.error.gated=this.derivative*i,this.error.responsibility=this.error.projected+this.error.gated}t=t||.1;for(var r in this.connections.inputs){var c=this.connections.inputs[r],h=this.error.projected*this.trace.elegibility[c.ID];for(var r in this.trace.extended){var s=this.neighboors[r];h+=s.error.responsibility*this.trace.extended[s.ID][c.ID]}c.weight+=t*h}this.bias+=t*this.error.responsibility},project:function(t,e){if(t==this)return this.selfconnection.weight=1,this.selfconnection;var i=this.connected(t);if(i&&"projected"==i.type)return"undefined"!=typeof e&&(i.connection.weight=e),i.connection;var r=new n.connection(this,t,e);this.connections.projected[r.ID]=r,this.neighboors[t.ID]=t,t.connections.inputs[r.ID]=r,t.trace.elegibility[r.ID]=0;for(var o in t.trace.extended){var s=t.trace.extended[o];s[r.ID]=0}return r},gate:function(t){this.connections.gated[t.ID]=t;var e=t.to;if(!(e.ID in this.trace.extended)){this.neighboors[e.ID]=e;var i=this.trace.extended[e.ID]={};for(var n in this.connections.inputs){var r=this.connections.inputs[n];i[r.ID]=0}}e.ID in this.trace.influences?this.trace.influences[e.ID].push(t):this.trace.influences[e.ID]=[t],t.gater=this},selfconnected:function(){return 0!==this.selfconnection.weight},connected:function(t){var e={type:null,connection:!1};if(this==t)return this.selfconnected()?(e.type="selfconnection",e.connection=this.selfconnection,e):!1;for(var i in this.connections)for(var n in this.connections[i]){var n=this.connections[i][n];if(n.to==t)return e.type=i,e.connection=n,e;if(n.from==t)return e.type=i,e.connection=n,e}return!1},clear:function(){for(var t in this.trace.elegibility)this.trace.elegibility[t]=0;for(var t in this.trace.extended)for(var e in this.trace.extended[t])this.trace.extended[t][e]=0;this.error.responsibility=this.error.projected=this.error.gated=0},reset:function(){this.clear();for(var t in this.connections)for(var e in this.connections[t])this.connections[t][e].weight=.2*Math.random()-.1;this.bias=.2*Math.random()-.1,this.old=this.state=this.activation=0},optimize:function(t,e){t=t||{};var i=[],r=[],o=[],s=t.memory||0,a=t.neurons||1,c=t.inputs||[],h=t.targets||[],u=t.outputs||[],p=t.variables||{},f=t.activation_sentences||[],l=t.trace_sentences||[],v=t.propagation_sentences||[],d=t.layers||{__count:0,__neuron:0},g=function(t){var i=e in d&&t[d.__count];i||(d.__count=t.push([])-1,d[e]=d.__count)};g(f),g(l),g(v);var y=d.__count,m=function(){var t=Array.prototype.slice.call(arguments);if(1==t.length){if("target"==t[0]){var e="target_"+h.length;h.push(s)}else var e=t[0];return e in p?p[e]:p[e]={value:0,id:s++}}var i=t.length>2;if(i)var n=t.pop();var r=t.shift(),o=t.pop();if(!i)var n=r[o];var e=o+"_";for(var a in t)e+=t[a]+"_";return e+=r.ID,e in p?p[e]:p[e]={value:n,id:s++}},w=function(){var t=Array.prototype.slice.call(arguments),e=t.pop(),i="";for(var n in t)i+="string"==typeof t[n]?t[n]:"F["+t[n].id+"]";e.push(i+";")},T=function(t){for(var e in t)if(t.hasOwnProperty(e))return!1;return!0},b=T(this.connections.projected),I=T(this.connections.gated),D="input"==e?!0:T(this.connections.inputs),O="output"==e?!0:b&&I,_=m("rate"),z=m(this,"activation");if(D)c.push(z.id);else{f[y].push(i),l[y].push(r),v[y].push(o);var j=m(this,"old"),L=m(this,"state"),E=m(this,"bias");if(this.selfconnection.gater)var x=m(this.selfconnection,"gain");if(this.selfconnected())var N=m(this.selfconnection,"weight");w(j," = ",L,i),this.selfconnected()?this.selfconnection.gater?w(L," = ",x," * ",N," * ",L," + ",E,i):w(L," = ",N," * ",L," + ",E,i):w(L," = ",E,i);for(var M in this.connections.inputs){var A=this.connections.inputs[M],S=m(A.from,"activation"),k=m(A,"weight");if(A.gater)var q=m(A,"gain");this.connections.inputs[M].gater?w(L," += ",S," * ",k," * ",q,i):w(L," += ",S," * ",k,i)}var F=m(this,"derivative");switch(this.squash){case n.squash.LOGISTIC:w(z," = (1 / (1 + Math.exp(-",L,")))",i),w(F," = ",z," * (1 - ",z,")",i);break;case n.squash.TANH:var P=m("aux"),R=m("aux_2");w(P," = Math.exp(",L,")",i),w(R," = 1 / ",P,i),w(z," = (",P," - ",R,") / (",P," + ",R,")",i),w(F," = 1 - (",z," * ",z,")",i);break;case n.squash.IDENTITY:w(z," = ",L,i),w(F," = 1",i);break;case n.squash.HLIM:w(z," = +(",L," > 0)",i),w(F," = 1",i)}for(var C in this.trace.extended){var H=this.trace.extended[C],U=this.neighboors[C],G=m("influences["+U.ID+"]"),B=m(U,"old"),Y=!1;U.selfconnection.gater==this&&(w(G," = ",B,r),Y=!0);for(var J in this.trace.influences[U.ID]){var X=m(this.trace.influences[U.ID][J],"weight"),W=m(this.trace.influences[U.ID][J].from,"activation");Y?w(G," += ",X," * ",W,r):(w(G," = ",X," * ",W,r),Y=!0)}}for(var M in this.connections.inputs){var A=this.connections.inputs[M];if(A.gater)var q=m(A,"gain");var S=m(A.from,"activation"),K=m(this,"trace","elegibility",A.ID,this.trace.elegibility[A.ID]);this.selfconnected()?this.selfconnection.gater?A.gater?w(K," = ",x," * ",N," * ",K," + ",q," * ",S,r):w(K," = ",x," * ",N," * ",K," + ",S,r):A.gater?w(K," = ",N," * ",K," + ",q," * ",S,r):w(K," = ",N," * ",K," + ",S,r):A.gater?w(K," = ",q," * ",S,r):w(K," = ",S,r);for(var C in this.trace.extended){var H=this.trace.extended[C],U=this.neighboors[C],G=m("influences["+U.ID+"]"),B=m(U,"old"),K=m(this,"trace","elegibility",A.ID,this.trace.elegibility[A.ID]),H=m(this,"trace","extended",U.ID,A.ID,this.trace.extended[U.ID][A.ID]);if(U.selfconnected())var V=m(U.selfconnection,"weight");if(U.selfconnection.gater)var Q=m(U.selfconnection,"gain");U.selfconnected()?U.selfconnection.gater?w(H," = ",Q," * ",V," * ",H," + ",F," * ",K," * ",G,r):w(H," = ",V," * ",H," + ",F," * ",K," * ",G,r):w(H," = ",F," * ",K," * ",G,r)}}for(var Z in this.connections.gated){var $=m(this.connections.gated[Z],"gain");w($," = ",z,i)}}if(!D){var tt=m(this,"error","responsibility",this.error.responsibility);if(O){var et=m("target");w(tt," = ",et," - ",z,o);for(var C in this.connections.inputs){var A=this.connections.inputs[C],K=m(this,"trace","elegibility",A.ID,this.trace.elegibility[A.ID]),k=m(A,"weight");w(k," += ",_," * (",tt," * ",K,")",o)}u.push(z.id)}else if(b||I){if(I){w(tt," = 0",o);for(var C in this.connections.projected){var Z=this.connections.projected[C],U=Z.to,it=m(Z,"weight"),nt=m(U,"error","responsibility",U.error.responsibility);if(Z.gater){var rt=m(Z,"gain");w(tt," += ",nt," * ",rt," * ",it,o)}else w(tt," += ",nt," * ",it,o)}w(tt," *= ",F,o);for(var C in this.connections.inputs){var A=this.connections.inputs[C],K=m(this,"trace","elegibility",A.ID,this.trace.elegibility[A.ID]),k=m(A,"weight");w(k," += ",_," * (",tt," * ",K,")",o)}}else if(b){w(tt," = 0",o);for(var C in this.trace.extended){var U=this.neighboors[C],G=m("aux"),B=m(U,"old");U.selfconnection.gater==this?w(G," = ",B,o):w(G," = 0",o);for(var A in this.trace.influences[U.ID]){var Z=this.trace.influences[U.ID][A],it=m(Z,"weight"),ot=m(Z.from,"activation");w(G," += ",it," * ",ot,o)}var nt=m(U,"error","responsibility",U.error.responsibility);w(tt," += ",nt," * ",G,o)}w(tt," *= ",F,o);for(var C in this.connections.inputs){var A=this.connections.inputs[C],st=m("aux");w(st," = 0",o);for(var C in this.trace.extended){var U=this.neighboors[C],nt=m(U,"error","responsibility",U.error.responsibility),H=m(this,"trace","extended",U.ID,A.ID,this.trace.extended[U.ID][A.ID]);w(st," += ",nt," * ",H,o)}var k=m(A,"weight");w(k," += ",_," * ",st,o)}}}else{var at=m("aux");for(var C in this.connections.projected){var Z=this.connections.projected[C],U=Z.to,it=m(Z,"weight"),nt=m(U,"error","responsibility",U.error.responsibility);if(Z.gater){var rt=m(Z,"gain");w(at," += ",nt," * ",rt," * ",it,o)}else w(at," += ",nt," * ",it,o)}var ct=m(this,"error","projected",this.error.projected);w(ct," = ",F," * ",at,o),w(at," = 0",o);for(var C in this.trace.extended){var U=this.neighboors[C],G=m("aux_2"),B=m(U,"old");U.selfconnection.gater==this?w(G," = ",B,o):w(G," = 0",o);for(var A in this.trace.influences[U.ID]){var Z=this.trace.influences[U.ID][A],it=m(Z,"weight"),ot=m(Z.from,"activation");w(G," += ",it," * ",ot,o)}var nt=m(U,"error","responsibility",U.error.responsibility);w(at," += ",nt," * ",G,o)}var ht=m(this,"error","gated",this.error.gated);w(ht," = ",F," * ",at,o),w(tt," = ",ct," + ",ht,o);for(var C in this.connections.inputs){var A=this.connections.inputs[C],st=m("aux"),K=m(this,"trace","elegibility",A.ID,this.trace.elegibility[A.ID]);w(st," = ",ct," * ",K,o);for(var C in this.trace.extended){var U=this.neighboors[C],nt=m(U,"error","responsibility",U.error.responsibility),H=m(this,"trace","extended",U.ID,A.ID,this.trace.extended[U.ID][A.ID]);w(st," += ",nt," * ",H,o)}var k=m(A,"weight");w(k," += ",_," * ",st,o)}}w(E," += ",_," * ",tt,o)}return{memory:s,neurons:a+1,inputs:c,outputs:u,targets:h,variables:p,activation_sentences:f,trace_sentences:l,propagation_sentences:v,layers:d}}},n.connection=function(t,e,i){if(!t||!e)throw"Connection Error: Invalid neurons";this.ID=n.connection.uid(),this.from=t,this.to=e,this.weight="undefined"==typeof i?.2*Math.random()-.1:i,this.gain=1,this.gater=null},n.squash={},n.squash.LOGISTIC=function(t,e){if(!e)return 1/(1+Math.exp(-t));var i=n.squash.LOGISTIC(t);return i*(1-i)},n.squash.TANH=function(t,e){if(e)return 1-Math.pow(n.squash.TANH(t),2);var i=Math.exp(t),r=1/i;return(i-r)/(i+r)},n.squash.IDENTITY=function(t,e){return e?1:t},n.squash.HLIM=function(t,e){return e?1:+(t>0)},function(){var t=0,e=0;n.uid=function(){return t++},n.connection.uid=function(){return e++},n.quantity=function(){return{neurons:t,connections:e}}}(),e&&(e.exports=n)},{}],6:[function(t,e,n){function r(t,e){e=e||{},this.network=t,this.rate=e.rate||.2,this.iterations=e.iterations||1e5,this.error=e.error||.005,this.cost=e.cost||null}r.prototype={train:function(t,e){function i(t){for(var e,i,n=t.length;n;e=Math.floor(Math.random()*n),i=t[--n],t[n]=t[e],t[e]=i);return t}var n,o,s,a,c=1,h=bucketSize=0,u=!1,p=e&&e.cost||this.cost||r.cost.MSE,f=Date.now();for(e&&(e.shuffle,e.iterations&&(this.iterations=e.iterations),e.error&&(this.error=e.error),e.rate&&(this.rate=e.rate),e.cost&&(this.cost=e.cost),e.schedule&&(this.schedule=e.schedule),e.customLog&&(console.log("Deprecated: use schedule instead of customLog"),this.schedule=e.customLog)),a=this.rate,Array.isArray(this.rate)&&(bucketSize=Math.floor(this.iterations/this.rate.length));!u&&h<this.iterations&&c>this.error;){if(c=0,bucketSize>0){var l=Math.floor(h/bucketSize);a=this.rate[l]||a}for(var v in t)n=t[v].input,s=t[v].output,o=this.network.activate(n),this.network.propagate(a,s),c+=p(s,o);h++,c/=t.length,e&&(this.schedule&&this.schedule.every&&h%this.schedule.every==0?u=this.schedule["do"]({error:c,iterations:h,rate:a}):e.log&&h%e.log==0&&console.log("iterations",h,"error",c,"rate",a),e.shuffle&&i(t))}var d={error:c,iterations:h,time:Date.now()-f};return d},test:function(t,e){var i,n,o,s=0,a=e&&e.cost||this.cost||r.cost.MSE,c=Date.now();for(var h in t)i=t[h].input,o=t[h].output,n=this.network.activate(i),s+=a(o,n);s/=t.length;var u={error:s,time:Date.now()-c};return u},workerTrain:function(t,e,i){function n(t){for(var e,i,n=t.length;n;e=Math.floor(Math.random()*n),i=t[--n],t[n]=t[e],t[e]=i);return t}function o(t){d.postMessage({action:"activate",input:t,memoryBuffer:c.network.optimized.memory},[c.network.optimized.memory.buffer])}function s(t){if(bucketSize>0){var e=Math.floor(u/bucketSize);a=c.rate[e]||a}d.postMessage({action:"propagate",target:t,rate:a,memoryBuffer:c.network.optimized.memory},[c.network.optimized.memory.buffer])}var a,c=this,h=1,u=bucketSize=0,p=t.length,f=!1,l=i&&i.cost||c.cost||r.cost.MSE,v=Date.now();i&&(i.shuffle,i.iterations&&(c.iterations=i.iterations),i.error&&(c.error=i.error),i.rate&&(c.rate=i.rate),i.cost&&(c.cost=i.cost),i.schedule&&(c.schedule=i.schedule),i.customLog&&(console.log("Deprecated: use schedule instead of customLog"),c.schedule=i.customLog)),a=c.rate,Array.isArray(c.rate)&&(bucketSize=Math.floor(c.iterations/c.rate.length));var d=c.network.worker();d.onmessage=function(r){c.network.optimized.ownership(r.data.memoryBuffer),"propagate"==r.data.action&&(g>=p?(g=0,u++,h/=t.length,i&&(c.schedule&&c.schedule.every&&u%c.schedule.every==0?f=c.schedule["do"]({error:h,iterations:u,rate:a}):i.log&&u%i.log==0&&console.log("iterations",u,"error",h),i.shuffle&&n(t)),!f&&u<c.iterations&&h>c.error?o(t[g].input):e({error:h,iterations:u,time:Date.now()-v}),h=0):o(t[g].input)),"activate"==r.data.action&&(h+=l(t[g].output,r.data.output),s(t[g].output),g++)};var g=0,u=0;o(t[g].input)},XOR:function(t){if(2!=this.network.inputs()||1!=this.network.outputs())throw"Error: Incompatible network (2 inputs, 1 output)";var e={iterations:1e5,log:!1,shuffle:!0,cost:r.cost.MSE};if(t)for(var i in t)e[i]=t[i];return this.train([{input:[0,0],output:[0]},{input:[1,0],output:[1]},{input:[0,1],output:[1]},{input:[1,1],output:[0]}],e)},DSR:function(t){t=t||{};for(var e=t.targets||[2,4,7,8],n=t.distractors||[3,5,6,9],o=t.prompts||[0,1],s=t.length||24,a=t.success||.95,c=t.iterations||1e5,h=t.rate||.1,u=t.log||0,p=t.schedule||{},f=t.cost||this.cost||r.cost.CROSS_ENTROPY,l=correct=i=j=success=0,v=1,d=e.length+n.length+o.length,g=function(t,e){var i=Math.random()*t|0,n=!1;for(var r in e)i==e[r]&&(n=!0);return n?g(t,e):i},y=function(t,e){for(var i in t)if(Math.round(t[i])!=e[i])return!1;return!0},m=Date.now();c>l&&(success<a||l%1e3!=0);){var w=[],T=s-o.length;for(i=0;i<T;i++){var b=Math.random()*n.length|0;w.push(n[b])}var I=[],D=[];for(i=0;i<o.length;i++)I.push(Math.random()*e.length|0),D.push(g(T,D));for(D=D.sort(),i=0;i<o.length;i++)w[D[i]]=e[I[i]],w.push(o[i]);var O=distractorsCorrect=0;for(v=0,i=0;i<s;i++){var _=[];for(j=0;j<d;j++)_[j]=0;_[w[i]]=1;var z=[];for(j=0;j<e.length;j++)z[j]=0;

if(i>=T){var L=i-T;z[I[L]]=1}var E=this.network.activate(_);y(E,z)?i<T?distractorsCorrect++:O++:this.network.propagate(h,z),v+=f(z,E),distractorsCorrect+O==s&&correct++}l%1e3==0&&(correct=0),l++;var x=l%1e3;x=0==x?1e3:x,success=correct/x,v/=s,u&&l%u==0&&console.log("iterations:",l," success:",success," correct:",correct," time:",Date.now()-m," error:",v),p["do"]&&p.every&&l%p.every==0&&p["do"]({iterations:l,success:success,error:v,time:Date.now()-m,correct:correct})}return{iterations:l,success:success,error:v,time:Date.now()-m}},ERG:function(t){t=t||{};var e=t.iterations||15e4,i=t.error||.05,n=t.rate||.1,o=t.log||500,s=t.cost||this.cost||r.cost.CROSS_ENTROPY,a=function(){this.paths=[]};a.prototype={connect:function(t,e){return this.paths.push({node:t,value:e}),this},any:function(){if(0==this.paths.length)return!1;var t=Math.random()*this.paths.length|0;return this.paths[t]},test:function(t){for(var e in this.paths)if(this.paths[e].value==t)return this.paths[e];return!1}};for(var c=function(){var t=new a,e=(new a).connect(t,"E"),i=(new a).connect(e,"S"),n=(new a).connect(e,"V").connect(i,"P"),r=(new a).connect(i,"X");r.connect(r,"S");var o=(new a).connect(n,"V");o.connect(o,"T"),i.connect(o,"X");var s=(new a).connect(r,"T").connect(o,"P"),c=(new a).connect(s,"B");return{input:c,output:t}},h=function(){var t=c(),e=c(),i=new a,n=(new a).connect(i,"E");t.output.connect(n,"T"),e.output.connect(n,"P");var r=(new a).connect(t.input,"P").connect(e.input,"T"),o=(new a).connect(r,"B");return{input:o,output:i}},u=function(){for(var t=h().input,e=t.any(),i="";e;)i+=e.value,e=e.node.any();return i},p=function(t){for(var e=h().input,i=0,n=t.charAt(i);i<t.length;){var r=e.test(n);if(!r)return!1;e=r.node,n=t.charAt(++i)}return!0},f=function(t,e){var i=0,n=-1,r=0,o=-1;for(var s in t)t[s]>i&&(i=t[s],n=s),e[s]>r&&(r=e[s],o=s);return n!=o},l=0,v=1,d={B:0,P:1,T:2,X:3,S:4,E:5},g=Date.now();e>l&&v>i;){var y=0;v=0;for(var m=u(),w=m.charAt(y),T=m.charAt(y+1);y<m.length-1;){for(var b=[],I=[],D=0;6>D;D++)b[D]=0,I[D]=0;b[d[w]]=1,I[d[T]]=1;var O=this.network.activate(b);f(O,I)&&this.network.propagate(n,I),w=m.charAt(++y),T=m.charAt(y+1),v+=s(I,O)}v/=m.length,l++,l%o==0&&console.log("iterations:",l," time:",Date.now()-g," error:",v)}return{iterations:l,error:v,time:Date.now()-g,test:p,generate:u}},timingTask:function(t){function e(t,e){for(var i=t+e,n=0,r=[],o=0;i>o;o++)r.push({input:[0,0],output:[0]});for(;i-20>n;){var s=Math.round(20*Math.random());r[n].input[0]=1;for(var a=n;n+s>=a;a++)r[a].input[1]=s/20,r[a].output[0]=.5;n+=s,s=Math.round(20*Math.random());for(var c=n+1;n+s>=c;c++)r[c].input[1]=r[n].input[1];n+=s}for(var h=[],u=[],p=0;i>p;p++)(t>p?h:u).push(r[p]);return{train:h,test:u}}if(2!=this.network.inputs()||1!=this.network.outputs())throw"Invalid Network: must have 2 inputs and one output";if("undefined"==typeof t)var t={};var i=t.iterations||200,n=t.error||.005,o=t.rate||[.03,.02],s=t.log===!1?!1:t.log||10,a=t.cost||this.cost||r.cost.MSE,c=t.trainSamples||7e3,h=t.trainSamples||1e3,u=e(c,h),p=this.train(u.train,{rate:o,log:s,iterations:i,error:n,cost:a});return{train:p,test:this.test(u.test)}}},r.cost={CROSS_ENTROPY:function(t,e){var i=0;for(var n in e)i-=t[n]*Math.log(e[n]+1e-15)+(1-t[n])*Math.log(1+1e-15-e[n]);return i},MSE:function(t,e){var i=0;for(var n in e)i+=Math.pow(t[n]-e[n],2);return i/e.length},BINARY:function(t,e){var i=0;for(var n in e)i+=Math.round(2*t[n])!=Math.round(2*e[n]);return i}},e&&(e.exports=r)},{}]},{},[1]);var Neuron = synaptic.Neuron, Layer = synaptic.Layer, Network = synaptic.Network, Trainer = synaptic.Trainer, Architect = synaptic.Architect;